/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.dialog;

import static org.eclipse.wb.swt.ResourceManager.decorateImage;
import static org.eclipse.wb.swt.ResourceManager.getPluginImage;
import static org.eclipse.wb.swt.SWTResourceManager.BOTTOM_RIGHT;

import java.util.ArrayList;
import java.util.List;

import org.conqat.ide.commons.ui.jface.TreeContentProviderBase;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.forms.widgets.FormToolkit;

/**
 * This dialog offers a common dialog to show multi-root tree structures and
 * allows user to select multiple items in the tree. It also offers an interface
 * to edit the tree on the fly.
 * 
 * @author mou
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 83D78C1AC95F8C254DFF26D4569C92C3
 */
public class ElementTreeMultiSelectDialog extends TitleAreaDialog {

	/** Default content provider */
	private static class EmptyTreeContentProvider extends
			TreeContentProviderBase {

		/** {@inheritDoc} */
		@Override
		public Object[] getChildren(Object parentElement) {
			return new Object[0];
		}
	}

	/** Label provider which adds selected icon on the original icon */
	private class DecorateLabelProvider extends LabelProvider {

		/** {@inheritDoc} */
		@Override
		public String getText(Object element) {
			return labelSupport.getText(element);
		}

		/** {@inheritDoc} */
		@Override
		public Image getImage(Object element) {
			Image image = labelSupport.getImage(element);
			if (!selectedElements.contains(element))
				return image;

			Image icon = getPluginImage("org.eclipse.ui",
					"/icons/full/ovr16/pinned_ovr.gif");
			if (icon == null)
				return image;

			return decorateImage(image, icon, BOTTOM_RIGHT);
		}
	}

	/** GUI Factory */
	private final FormToolkit formToolkit = new FormToolkit(
			Display.getDefault());

	/** editing module */
	private IDialogEditSupport dialogEditSupport;

	/** roots of the tree */
	private ArrayList<Object> rootElements;

	/** selected element */
	private ArrayList<Object> selectedElements;

	/** content provider for the roots */
	private ITreeContentProvider contentSupport;

	/** label provider for the elements */
	private ILabelProvider labelSupport;

	/** left tree viewer */
	private TreeViewer elementTreeViewer;

	/** right tree viewer */
	private TreeViewer selectTreeViewer;

	/** Constructor */
	public ElementTreeMultiSelectDialog(Shell parentShell, List<?> roots,
			List<?> selected, ITreeContentProvider contentProvider,
			ILabelProvider labelProvider, IDialogEditSupport edit) {
		super(parentShell);

		if (roots == null)
			rootElements = new ArrayList<Object>();
		else
			rootElements = new ArrayList<Object>(roots);

		if (selected == null)
			selectedElements = new ArrayList<Object>();
		else
			selectedElements = new ArrayList<Object>(selected);

		dialogEditSupport = edit;
		contentSupport = contentProvider;
		labelSupport = labelProvider;
	}

	/** {@inheritDoc} */
	@Override
	protected Control createDialogArea(Composite parent) {
		// CAUTION: The code below is automatically generated, do not edit
		// manually!

		setHelpAvailable(false);

		setTitle("Select");
		setMessage("Select one or more elements");

		// create controls
		Composite area = (Composite) super.createDialogArea(parent);
		Composite container = new Composite(area, SWT.NONE);

		GridLayout gl_container = new GridLayout(4, false);
		gl_container.verticalSpacing = 2;
		gl_container.marginRight = 5;
		gl_container.marginLeft = 5;
		gl_container.marginBottom = 5;
		gl_container.marginTop = 5;
		gl_container.marginWidth = 0;
		gl_container.marginHeight = 0;
		container.setLayout(gl_container);
		container.setLayoutData(new GridData(GridData.FILL_BOTH));

		Composite lefttoolbar = new Composite(container, SWT.NONE);
		lefttoolbar.setLayoutData(new GridData(SWT.CENTER, SWT.FILL, false,
				true, 1, 1));

		GridLayout gl_lefttoolbar = new GridLayout(1, false);
		gl_lefttoolbar.verticalSpacing = 2;
		gl_lefttoolbar.horizontalSpacing = 2;
		gl_lefttoolbar.marginHeight = 0;
		gl_lefttoolbar.marginWidth = 0;
		lefttoolbar.setLayout(gl_lefttoolbar);

		Button createButton = new Button(lefttoolbar, SWT.NONE);
		createButton.setImage(getPluginImage("org.eclipse.ui",
				"/icons/full/obj16/add_obj.gif"));
		formToolkit.adapt(createButton, true, true);

		Button deleteButton = new Button(lefttoolbar, SWT.NONE);
		deleteButton.setImage(getPluginImage("org.eclipse.ui",
				"/icons/full/obj16/delete_obj.gif"));
		formToolkit.adapt(deleteButton, true, true);

		Button editButton = new Button(lefttoolbar, SWT.NONE);
		editButton.setImage(getPluginImage("org.eclipse.ui",
				"/icons/full/etool16/editor_area.gif"));
		formToolkit.adapt(editButton, true, true);

		elementTreeViewer = new TreeViewer(container, SWT.BORDER);
		Tree elementTree = elementTreeViewer.getTree();
		elementTree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true,
				1, 1));

		Composite toolbarbox = new Composite(container, SWT.NONE);
		GridLayout gl_toolbarbox = new GridLayout(1, false);

		gl_toolbarbox.verticalSpacing = 2;
		gl_toolbarbox.horizontalSpacing = 2;
		gl_toolbarbox.marginHeight = 0;
		gl_toolbarbox.marginWidth = 0;
		toolbarbox.setLayout(gl_toolbarbox);
		toolbarbox.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, false,
				true, 1, 1));

		Button selectButton = new Button(toolbarbox, SWT.NONE);
		selectButton.setText(">>");

		Button deselectButton = new Button(toolbarbox, SWT.NONE);
		deselectButton.setText("<<");

		selectTreeViewer = new TreeViewer(container, SWT.BORDER);
		Tree selectTree = selectTreeViewer.getTree();
		selectTree.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true,
				1, 1));

		// create listeners
		createButton.addSelectionListener(new SelectionAdapter() {
			/** {@inheritDoc} */
			@Override
			public void widgetSelected(SelectionEvent e) {
				createElement();
			}
		});

		deleteButton.addSelectionListener(new SelectionAdapter() {
			/** {@inheritDoc} */
			@Override
			public void widgetSelected(SelectionEvent e) {
				deleteElement();
			}
		});

		editButton.addSelectionListener(new SelectionAdapter() {
			/** {@inheritDoc} */
			@Override
			public void widgetSelected(SelectionEvent e) {
				editElement();
			}
		});

		selectButton.addSelectionListener(new SelectionAdapter() {
			/** {@inheritDoc} */
			@Override
			public void widgetSelected(SelectionEvent e) {
				selectElement();
			}
		});

		deselectButton.addSelectionListener(new SelectionAdapter() {
			/** {@inheritDoc} */
			@Override
			public void widgetSelected(SelectionEvent e) {
				deselectElement();
			}
		});

		elementTreeViewer.addDoubleClickListener(new IDoubleClickListener() {
			@Override
			public void doubleClick(DoubleClickEvent event) {
				selectElement();
			}
		});

		selectTreeViewer.addDoubleClickListener(new IDoubleClickListener() {
			@Override
			public void doubleClick(DoubleClickEvent event) {
				deselectElement();
			}
		});

		// label, content and input
		elementTreeViewer.setLabelProvider(new DecorateLabelProvider());
		selectTreeViewer.setLabelProvider(new DecorateLabelProvider());

		elementTreeViewer.setContentProvider(new MultiRootTreeContentProvider(
				rootElements, contentSupport));
		selectTreeViewer.setContentProvider(new MultiRootTreeContentProvider(
				selectedElements, new EmptyTreeContentProvider()));

		elementTreeViewer.setInput(rootElements);
		selectTreeViewer.setInput(selectedElements);

		// hide edit toolbar if no edit function is available
		if (dialogEditSupport == null) {
			lefttoolbar.setVisible(false);
			((GridData) lefttoolbar.getLayoutData()).exclude = true;
		}

		return area;
	}

	/** {@inheritDoc} */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				false);
		createButton(parent, IDialogConstants.CANCEL_ID, "Cancel", true);
	}

	/** {@inheritDoc} */
	@Override
	protected Point getInitialSize() {
		return new Point(640, 480);
	}

	/** Returns configuration. */
	public IDialogEditSupport getEditSupport() {
		return dialogEditSupport;
	}

	/** Returns rootElements. */
	public List<Object> getRootElements() {
		return rootElements;
	}

	/** Returns selectedElements. */
	public List<Object> getSelectedElements() {
		return selectedElements;
	}

	/** Returns contentSupport. */
	public IStructuredContentProvider getContentProvider() {
		return contentSupport;
	}

	/** Returns labelSupport. */
	public ILabelProvider getLabelProvider() {
		return labelSupport;
	}

	/**
	 * create an element by using the editing module and update the left tree
	 * viewer
	 */
	private void createElement() {
		if (dialogEditSupport == null)
			return;

		try {
			ITreeSelection selection = (ITreeSelection) elementTreeViewer
					.getSelection();
			if (selection.getPaths() != null
					&& selection.getPaths().length != 1)
				return; // at most 1 element can be selected

			// create the new element
			Object o;
			if (selection.getPaths() == null) {
				o = dialogEditSupport.createElement(null);
			} else {
				o = dialogEditSupport.createElement(selection.getPaths()[0]
						.getLastSegment());
			}
			if (o == null)
				return;

			TreePath path;
			if (selection.getPaths() == null) {
				path = new TreePath(new Object[] { rootElements });
				rootElements.add(o);
			} else {
				path = selection.getPaths()[0];
			}

			// insert the new element
			elementTreeViewer.add(path, o);
		} catch (Exception e2) {
			e2.printStackTrace();
		}
	}

	/** Call delete function on the selected elements */
	private void deleteElement() {
		if (dialogEditSupport == null)
			return;

		try {
			// find the path to be deleted
			ITreeSelection selection = (ITreeSelection) elementTreeViewer
					.getSelection();
			if (selection.getPaths() == null)
				return;

			// delete selected elements
			for (TreePath path : selection.getPaths()) {
				if (dialogEditSupport.deleteElement(path.getLastSegment())) {
					if (rootElements.contains(path.getLastSegment()))
						rootElements.remove(path.getLastSegment());

					elementTreeViewer.remove(path);

					for (Object selected : selectedElements.toArray()) {
						if (isParent(path.getLastSegment(), selected)) {
							selectedElements.remove(selected);
							selectTreeViewer.remove(selected);
						}
					}
				}
			}
		} catch (Exception e2) {
			e2.printStackTrace();
		}
	}

	/**
	 * Check whether the given root element is a direct or indirect parent of
	 * the leaf element
	 * 
	 * @param root
	 *            the parent
	 * @param leaf
	 *            the child
	 * @return true if the root element is a direct or indirect parent of leaf
	 *         element, otherwise false
	 */
	private boolean isParent(Object root, Object leaf) {
		if (root == leaf) {
			return true;
		}

		Object parent = contentSupport.getParent(leaf);

		while (parent != null) {
			if (parent == root) {
				return true;
			}
			parent = contentSupport.getParent(parent);
		}

		return false;
	}

	/** Call edit function on the selected element */
	private void editElement() {
		if (dialogEditSupport == null) {
			return;
		}

		try {
			// find the path to the element
			ITreeSelection selection = (ITreeSelection) elementTreeViewer
					.getSelection();
			if (selection.getPaths() == null
					|| selection.getPaths().length != 1)
				return; // 1 element must be selected

			TreePath path = selection.getPaths()[0];
			dialogEditSupport.editElement(path.getLastSegment());

			elementTreeViewer.refresh();
			selectTreeViewer.refresh();
		} catch (Exception e2) {
			e2.printStackTrace();
		}
	}

	/** Add selected elements to the select list. */
	private void selectElement() {
		try {
			// find the path to the element
			ITreeSelection selection = (ITreeSelection) elementTreeViewer
					.getSelection();
			if (selection.getPaths() == null)
				return;

			for (TreePath path : selection.getPaths()) {
				if (selectedElements.contains(path.getLastSegment()))
					continue;

				selectedElements.add(path.getLastSegment());
				selectTreeViewer.add(selectedElements, path.getLastSegment());
				elementTreeViewer.update(path.getLastSegment(), null);
			}

		} catch (Exception e2) {
			e2.printStackTrace();
		}
	}

	/** Remove selected elements from the select list */
	private void deselectElement() {
		try {
			// find the path to the element
			ITreeSelection selection = (ITreeSelection) selectTreeViewer
					.getSelection();
			if (selection.getPaths() == null)
				return;

			for (TreePath path : selection.getPaths()) {
				if (selectedElements.remove(path.getLastSegment())) {
					selectTreeViewer.remove(path.getLastSegment());
					elementTreeViewer.update(path.getLastSegment(), null);
				}
			}

		} catch (Exception e2) {
			e2.printStackTrace();
		}
	}
}