/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.fieldassist;

import org.eclipse.jface.bindings.keys.KeyStroke;
import org.eclipse.jface.bindings.keys.ParseException;
import org.eclipse.jface.fieldassist.ContentProposalAdapter;
import org.eclipse.jface.fieldassist.IContentProposal;
import org.eclipse.jface.fieldassist.IContentProposalListener;
import org.eclipse.jface.fieldassist.SimpleContentProposalProvider;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.swt.widgets.Text;

/**
 * This class provides easy access to auto completion in {@link Text} fields.
 * Unlike the original this implementation takes care of proper in place
 * insertion of proposals and provides ability to dynamically provide context
 * sensitive proposals.
 * 
 * This class is not yet thoroughly tested.
 * 
 * @author doebber
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating RED Hash:
 */
public class FieldAssist {

	/** The text field to assist. */
	private final Text field;

	/**
	 * ProposalProvider wrapper class that takes care of proper insertion. This
	 * is intended to be transparent to the implementor.
	 */
	private final AF3ContentProposalProvider provider;

	/** The keystroke the field assist is activated for. */
	private KeyStroke keystroke;

	/**
	 * Default keystroke for assist activation. Used if no other keystroke
	 * specified.
	 */
	public static final String DEFAULT_KEYSTROKE = "Ctrl+Space";

	/**
	 * Array of chars the assist is activated for. A typical example in Java
	 * would be '.' as an auto activation char.
	 */
	private char[] autoActivationChars;

	/**
	 * Constructor. Sufficient for most applications. Initializes with default
	 * keystroke and no auto activation chars.
	 */
	public FieldAssist(Text field, IProposalProvider provider) {
		this.field = field;
		this.provider = new AF3ContentProposalProvider(provider);
		initialize();
	}

	/** Constructor. Provides additional parameters which may be null. */
	public FieldAssist(Text field, IProposalProvider provider,
			KeyStroke keystroke, char[] autoActivationChars) {
		this.field = field;
		this.provider = new AF3ContentProposalProvider(provider);
		this.keystroke = keystroke;
		this.autoActivationChars = autoActivationChars;
		initialize();
	}

	/** Initializes the FieldAssist object. */
	private void initialize() {
		provider.setFiltering(true);
		if (keystroke == null) {
			try {
				keystroke = KeyStroke.getInstance(DEFAULT_KEYSTROKE);
			} catch (ParseException e) {
				// Should not happen
				e.printStackTrace();
			}
		}
		ContentProposalAdapter adapter = new ContentProposalAdapter(field,
				new TextContentAdapter(), provider, keystroke,
				autoActivationChars);
		adapter.setPropagateKeys(true);
		adapter.setProposalAcceptanceStyle(ContentProposalAdapter.PROPOSAL_IGNORE);
		adapter.addContentProposalListener(new IContentProposalListener() {

			@Override
			/** {@inheritDoc} */
			public void proposalAccepted(IContentProposal proposal) {
				int end = field.getCaretPosition();
				int start = findBeginningOfWord(field.getText(), end);
				String newContents = field.getText().substring(0,
						Math.max(0, start + 1));
				newContents = newContents.concat(proposal.getContent());
				newContents = newContents.concat(field.getText().substring(
						Math.min(field.getText().length(), end)));
				field.setText(newContents);
				field.setSelection(start + proposal.getContent().length() + 1);

			}
		});
	}

	/**
	 * Wrapper class transparent to the implementor that takes care of proper in
	 * place insertion.
	 */
	private final class AF3ContentProposalProvider extends
			SimpleContentProposalProvider {

		/**
		 * Stores the custom proposal provider implemented for the concrete
		 * application.
		 */
		private IProposalProvider concreteProvider;

		/**
		 * Constructor for static proposals. Is never called but required by the
		 * compiler.
		 */
		public AF3ContentProposalProvider(String[] proposals) {
			super(proposals);
			this.concreteProvider = new StaticProposalProvider(proposals);
		}

		/** Constructor. */
		public AF3ContentProposalProvider(IProposalProvider provider) {
			super(new String[0]);
			this.concreteProvider = provider;
		}

		/** {@inheritDoc} */
		@Override
		public IContentProposal[] getProposals(String contents, int position) {
			int tmp = findBeginningOfWord(contents, position);
			String currentWord = contents.substring(
					Math.min(position, tmp + 1), position);
			setProposals(concreteProvider.getProposals(currentWord));
			return super.getProposals(currentWord, position);
		}
	}

	/** Finds the beginning of the word the cursor is placed in. */
	private int findBeginningOfWord(String contents, int position) {
		int tmp = Math.max(0, position - 1);
		while (tmp > 0 && !Character.isWhitespace(contents.charAt(tmp))) {
			// TODO include operators like '+' to be handled as whitespace
			tmp--;
		}
		return tmp;
	}
}
