/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.utils;

import java.awt.Polygon;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.fortiss.tooling.base.layout.DefaultLayoutConstants;
import org.fortiss.tooling.base.model.layout.EOrientation;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.model.layout.OffsetOrientation;
import org.fortiss.tooling.base.ui.dnd.gef.ElementDropContext;
import org.fortiss.tooling.base.utils.LayoutModelElementFactory;

/**
 * Utility class for rectangle-shaped edit part layout.
 * 
 * @author wandinger
 * @author hummel
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 3DAA7FF0B916B47CC06441E6AC6808BC
 */
public class RectangleLayoutUtils {

	/**
	 * Returns the {@link OffsetOrientation} of the point on the rectangle
	 * border having a given Point inside the rectangle.
	 */
	public static OffsetOrientation calculateRectangleBorderOrientation(
			Point source, Dimension rectangleSize, int inset) {
		int offset = 0;
		EOrientation orientation = EOrientation.NORTH;

		// The figure bounds
		Rectangle bounds = new Rectangle(0, 0, rectangleSize.width,
				rectangleSize.height);

		// Divide the rectangle into four diagonal sectors
		Polygon northSector = new Polygon(new int[] { bounds.getCenter().x,
				bounds.x, bounds.width }, new int[] { bounds.getCenter().y,
				bounds.y, bounds.y }, 3);
		Polygon eastSector = new Polygon(new int[] { bounds.getCenter().x,
				bounds.width, bounds.width }, new int[] { bounds.getCenter().y,
				bounds.y, bounds.height }, 3);
		Polygon southSector = new Polygon(new int[] { bounds.getCenter().x,
				bounds.width, bounds.x }, new int[] { bounds.getCenter().y,
				bounds.height, bounds.height }, 3);
		Polygon westSector = new Polygon(new int[] { bounds.getCenter().x,
				bounds.x, bounds.x }, new int[] { bounds.getCenter().y,
				bounds.height, bounds.y }, 3);

		// Determine in which sector we are and clamp the position accordingly
		if (northSector.contains(source.x, source.y)) {
			orientation = EOrientation.NORTH;
			offset = (int) (source.x * (1 - (2 * inset) / (double) bounds.width));
		}
		if (westSector.contains(source.x, source.y)) {
			orientation = EOrientation.WEST;
			offset = (int) (source.y * (1 - (2 * inset)
					/ (double) bounds.height));
		}
		if (southSector.contains(source.x, source.y)) {
			orientation = EOrientation.SOUTH;
			offset = (int) (source.x * (1 - (2 * inset) / (double) bounds.width));
		}
		if (eastSector.contains(source.x, source.y)) {
			orientation = EOrientation.EAST;
			offset = (int) (source.y * (1 - (2 * inset)
					/ (double) bounds.height));
		}
		return LayoutModelElementFactory.createOffsetOrientation(offset,
				orientation, null);
	}

	/** Ensure that the minimum size of a rectangular shape is redeemed. */
	public static Dimension clampRectangularShapeSize(Dimension oldSize,
			int minimumWidth, int minimumHeight) {
		// check for minimum size
		Dimension newSize = new Dimension(
				oldSize.width < minimumWidth ? minimumWidth : oldSize.width,
				oldSize.height < minimumHeight ? minimumHeight : oldSize.height);
		return newSize;
	}

	/** Ensure that the minimum size of a rectangular shape is redeemed. */
	public static Dimension retainMinimumRectangularShapeSize(Dimension oldSize) {
		return clampRectangularShapeSize(oldSize,
				DefaultLayoutConstants.DEFAULT_SHAPE_MINIMUM_WIDTH,
				DefaultLayoutConstants.DEFAULT_SHAPE_MINIMUM_HEIGHT);
	}

	/** Layouts the given connector within the surrounding rectangle. */
	public static void layoutConnectorInRectangle(
			ILayoutedModelElement layouted, ILayoutedModelElement container,
			Point location) {
		Point position = location.getCopy();
		org.fortiss.tooling.base.model.layout.Point np = LayoutDataUtils
				.getNodePosition(container);
		position.translate(-np.getX(), -np.getY());
		// do border clamping
		org.fortiss.tooling.base.model.layout.Dimension nd = LayoutDataUtils
				.getNodeSize(container);
		Dimension d = new Dimension(nd.getWidth(), nd.getHeight());
		OffsetOrientation orient = RectangleLayoutUtils
				.calculateRectangleBorderOrientation(position, d,
						DefaultLayoutConstants.DEFAULT_SHAPE_INSETS);
		// snap to grid and store the value in the model
		orient.setOffset(SnapToGridUtils.snapToGrid(orient.getOffset()));
		LayoutDataUtils.setConnectorOffsetOrientation(layouted, orient);
	}

	/** Layouts the given layouted object using the position in the context. */
	public static void layoutNodeInRoot(ILayoutedModelElement layouted,
			ElementDropContext context, int positionOffset, boolean snapToGrid) {
		layoutNodeInRoot(layouted, context.getPosition(), positionOffset,
				snapToGrid);
	}

	/** Layouts the given layouted object using the position in the context. */
	public static void layoutNodeInRoot(ILayoutedModelElement layouted,
			Point location, int positionOffset, boolean snapToGrid) {
		Point position = location.getCopy();
		position.translate(-positionOffset, -positionOffset);
		if (snapToGrid) {
			SnapToGridUtils.snapToGrid(position);
		}
		LayoutDataUtils.setNodePosition(layouted, position.x, position.y);
	}

	/** Returns the bounds rectangle of the connector given its parent bounds. */
	public static Rectangle getAbsoluteConnectorBounds(
			ILayoutedModelElement connector, ILayoutedModelElement parent) {
		Rectangle result = new Rectangle(0, 0,
				DefaultLayoutConstants.DEFAULT_CONNECTOR_SIZE,
				DefaultLayoutConstants.DEFAULT_CONNECTOR_SIZE);

		// get orientation
		OffsetOrientation orient = LayoutDataUtils
				.getConnectorOffsetOrientation(connector);
		// get parent size
		org.fortiss.tooling.base.model.layout.Dimension parentSize = LayoutDataUtils
				.getNodeSize(parent);
		org.fortiss.tooling.base.model.layout.Point pos = LayoutDataUtils
				.getAbsolute(orient, parentSize,
						DefaultLayoutConstants.DEFAULT_SHAPE_INSETS);
		int relX = pos.getX();
		int relY = pos.getY();
		org.fortiss.tooling.base.model.layout.Point parentPos = LayoutDataUtils
				.getNodePosition(parent);
		// fix X coordinate
		if (relX > parentSize.getWidth()
				- DefaultLayoutConstants.DEFAULT_CONNECTOR_SIZE) {
			relX = parentSize.getWidth()
					- DefaultLayoutConstants.DEFAULT_CONNECTOR_SIZE;
		}
		// fix Y coordinate
		if (relY > parentSize.getHeight()
				- DefaultLayoutConstants.DEFAULT_CONNECTOR_SIZE) {
			relY = parentSize.getHeight()
					- DefaultLayoutConstants.DEFAULT_CONNECTOR_SIZE;
		}

		result.x = parentPos.getX() + relX;
		result.y = parentPos.getY() + relY;
		return result;
	}
}
