/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.actions;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.ActionFactory;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.ICommandStackService;
import org.fortiss.tooling.kernel.service.IConnectionCompositorService;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.ui.extension.base.EObjectActionBase;

/**
 * The {@link DeleteAction} uses the {@link IElementCompositorService} and the
 * {@link IConnectionCompositorService} to determine the enabled state of this
 * action.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 2EE81A5B2E8BB3E3ADEBD8C416F749EE
 */
public class DeleteAction extends EObjectActionBase<EObject> {

	/** Flag for using element or connection composition service. */
	private boolean useElementCompositor;

	/** Constructor. */
	public DeleteAction() {
		super("Delete", PlatformUI.getWorkbench().getSharedImages()
				.getImageDescriptor(ISharedImages.IMG_TOOL_DELETE));
		setId(ActionFactory.DELETE.getId());
	}

	/** {@inheritDoc} */
	@Override
	public void run() {
		if (getTarget() == null) {
			return;
		}
		if (IPersistencyService.INSTANCE.isTopLevelElement(getTarget())) {
			doTopLevelElementDelete();
		} else {
			ICommandStackService.INSTANCE.runAsCommand(getTarget(),
					new Runnable() {
						@Override
						public void run() {
							doCompositorBasedDelete();
						}
					});
		}
	}

	/** User interaction before deleting a top-level element. */
	private void doTopLevelElementDelete() {
		ITopLevelElement topElement = IPersistencyService.INSTANCE
				.getTopLevelElementFor(getTarget());
		Shell sh = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
				.getShell();
		String message = "Are you sure you want to delete '"
				+ topElement.getSaveableName() + "'?";

		final MessageDialog dialog = new MessageDialog(sh, "Confirm Delete",
				null, message, MessageDialog.QUESTION,
				new String[] { IDialogConstants.YES_LABEL,
						IDialogConstants.NO_LABEL }, 0) {
			@Override
			protected int getShellStyle() {
				return super.getShellStyle() | SWT.SHEET;
			}
		};

		sh.getDisplay().syncExec(new Runnable() {
			@Override
			public void run() {
				dialog.open();
			}
		});

		if (dialog.getReturnCode() == 0) {
			topElement.delete();
		}
	}

	/** Computes enabled state of the delete action. */
	@Override
	protected boolean computeEnabled() {
		EObject element = getTarget();
		if (!IPersistencyService.INSTANCE.isTopLevelElement(element)) {
			// use composition services for deletion
			if (IElementCompositorService.INSTANCE.canDecompose(element)) {
				useElementCompositor = true;
				return true;
			} else if (IConnectionCompositorService.INSTANCE
					.canDisconnect(element)) {
				useElementCompositor = false;
				return true;
			}
			return false;
		}
		// use top-level element deletion mechanism
		ITopLevelElement topElement = IPersistencyService.INSTANCE
				.getTopLevelElementFor(element);
		return topElement.canDelete();
	}

	/** Executes the delete using the correct service. */
	private void doCompositorBasedDelete() {
		EObject target = getTarget();
		if (useElementCompositor) {
			IElementCompositorService.INSTANCE.decompose(target);
		} else {
			IConnectionCompositorService.INSTANCE.disconnect(target);
		}
	}
}