/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.util;

import java.util.Collection;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.kernel.extension.data.IElementCompositionContext;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.ui.dnd.CompositionServiceLocalCopyPasteTransfer;

/**
 * Methods for copy/paste transfer. EMF serialization and {@link TextTransfer}
 * is used to transfer an {@link EObject} to and from the clipboard.
 * 
 * @author hoelzl
 * @author hummel
 * 
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 20CBD0E8F2561401E2A9300A5A41759C
 */
public class CopyPasteUtils {

	/**
	 * Copies the given selected objects to the clipboard.
	 * 
	 * @param selection
	 *            the collection of {@link EObject}s to be copyied to the
	 *            clipboard.
	 */
	public static void copyToClipboard(Collection<EObject> selection) {
		try {
			EObject[] content = new EObject[selection.size()];
			int i = 0;
			for (EObject sel : selection) {
				content[i] = EcoreUtil.copy(sel);
			}
			Clipboard clipboard = new Clipboard(Display.getDefault());
			clipboard.setContents(new Object[] { content },
					new Transfer[] { CompositionServiceLocalCopyPasteTransfer
							.getInstance() }, DND.CLIPBOARD
							| DND.SELECTION_CLIPBOARD);
			clipboard.dispose();
		} catch (Exception e) {
			MessageDialog.openError(Display.getDefault().getActiveShell(),
					"Copy",
					"Clipboard contents not writeable: " + e.getMessage());
			return;
		}
	}

	/** Returns elements read from clipboard. */
	private static EObject[] getClipBoardContent() {
		Clipboard clipboard = new Clipboard(Display.getDefault());

		EObject[] content = (EObject[]) clipboard
				.getContents(CompositionServiceLocalCopyPasteTransfer
						.getInstance());
		clipboard.dispose();
		return content;
	}

	/**
	 * Attempts to paste the contents of the clipboard into the given target
	 * object. The given element composition context is forwarded to the element
	 * composition service.
	 * 
	 * @param target
	 *            the target element to be passed to the compositor
	 * @param context
	 *            the composition context to be passed to the compositor
	 */
	public static void pasteFromClipboard(EObject target,
			IElementCompositionContext context) {
		if (!canPasteInto(target, context)) {
			return;
		}
		for (EObject insertObj : getClipBoardContent()) {
			IElementCompositorService.INSTANCE.compose(target,
					EcoreUtil.copy(insertObj), context);
		}
	}

	/**
	 * Returns whether the current clipboard element is pasteable into the given
	 * target using the given composition context.
	 * 
	 * @param target
	 *            the target element to be passed to the compositor
	 * @param context
	 *            the composition context to be passed to the compositor
	 * @return true if the clipboard element can be composed with the given
	 *         target
	 */
	public static boolean canPasteInto(EObject target,
			IElementCompositionContext context) {
		EObject[] contents = getClipBoardContent();
		if (contents == null) {
			return false;
		}
		for (EObject insertObj : contents) {
			if (IElementCompositorService.INSTANCE.canCompose(target,
					insertObj, context)) {
				return true;
			}
		}
		return false;
	}
}
